#import "AlertTestsViewController.h"
#import "AlertTestsAppDelegate.h"

@implementation AlertTestsViewController
@synthesize shape, lockButton, undoManager, lockIsLocked, undoRedoMoveAlert;


- (void)viewWillAppear:(BOOL)animated
	{
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	CGRect theStartRect = CGRectMake([defaults floatForKey:OriginX], shape.frame.origin.y, shape.frame.size.width, shape.frame.size.height);
	[shape setFrame:theStartRect];
	lockIsLocked = NO;

	[defaults addObserver:self forKeyPath:OriginX options:(NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld) context:NULL];
	undoManager = [[NSUndoManager alloc] init];
	
	[super viewWillAppear:animated];
	}

- (BOOL)canBecomeFirstResponder
	{
	return YES;
	}
	
- (void)viewDidAppear:(BOOL)animated
	{
	[self becomeFirstResponder];
	[super viewDidAppear:animated];
	}

- (void)dealloc
	{
	[shape release];
	[lockButton release];
	[undoManager release];
	[undoRedoMoveAlert release];
	[[NSUserDefaults standardUserDefaults] removeObserver:self forKeyPath:OriginX];
	[super dealloc];
	}


#pragma mark -
#pragma mark ACTION METHODS

- (IBAction)toggleLock
	{
	//Reverse Bool Value
	lockIsLocked = !lockIsLocked;
	
	if (lockIsLocked == YES)
		{
		[lockButton setTitle:@"Unlock" forState:UIControlStateNormal];
		shape.alpha = 0.5;
		}
		else
		{
		[lockButton setTitle:@"Lock" forState:UIControlStateNormal];
		shape.alpha = 1.0;
		}
	}


#pragma mark -
#pragma mark TOUCH METHODS

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
	{
	if (lockIsLocked == NO)
		{
		CGPoint point = [[touches anyObject] locationInView:self.view];
		CGRect frame = [shape frame];
		
		frame.origin.x = point.x;
		[shape setFrame:frame];
		}
	}
	
- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
	{
	if (lockIsLocked == NO)
		{
		[[NSUserDefaults standardUserDefaults] setFloat:shape.frame.origin.x forKey:OriginX];
		}
	}
	

#pragma mark -
#pragma mark UNDO/REDO METHODS

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
	{
	if([keyPath isEqualToString:OriginX])
		{
		//Setup Undo Move
		redoMoveFloat = [[change objectForKey:NSKeyValueChangeNewKey] floatValue];
		undoMoveFloat = [[change objectForKey:NSKeyValueChangeOldKey] floatValue];
		[[undoManager prepareWithInvocationTarget:self] undoOrRedoMove:@"undo" toPosition:undoMoveFloat withOpposingMoveValue:redoMoveFloat];
		[undoManager setActionName:[NSString stringWithFormat:@"Move"]];
		}
	}
	
- (void)undoOrRedoMove:(NSString *)toggle toPosition:(float)position withOpposingMoveValue:(float)redoValue
	{
	if (lockIsLocked == YES)
		{
		undoRedoMoveAlert = [[UIAlertView alloc]	initWithTitle:[NSString stringWithFormat:@"Locked"]
									message:nil  
									delegate:self 
									cancelButtonTitle:[NSString stringWithFormat:@"Cancel"] 
									otherButtonTitles:[NSString stringWithFormat:@"Unlock"], nil];
		[undoRedoMoveAlert show];
		[undoRedoMoveAlert release];
		}
		else
		{
		//Toggle Undo Or Redo
		if ([toggle isEqualToString:@"undo"])
			{
			//Setup Redo Move
			[[undoManager prepareWithInvocationTarget:self] undoOrRedoMove:nil toPosition:redoMoveFloat withOpposingMoveValue:undoMoveFloat];
			[undoManager setActionName:[NSString stringWithFormat:@"Move"]];
			}

		//Animate Undo Or Redo Movement
		[UIView beginAnimations:@"undoOrRedoMove" context:NULL];	
		[UIView setAnimationCurve:UIViewAnimationCurveEaseInOut];
		[UIView setAnimationDuration:0.5];

		CGRect frame = [shape frame];
		frame.origin.x = position;
		[shape setFrame:frame];

		[UIView commitAnimations];
		
		//Save New Position As Default
		[[NSUserDefaults standardUserDefaults] setFloat:position forKey:OriginX];
		}
	}

- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
	{
	if (alertView == undoRedoMoveAlert)
		{
		if (buttonIndex == 1)
			{
			NSLog(@"Unlock Button Pressed");
			[self toggleLock];

			if ([undoManager isUndoing])
				[self undoOrRedoMove:@"undo" toPosition:undoMoveFloat withOpposingMoveValue:redoMoveFloat];
			else if ([undoManager isRedoing])
				[self undoOrRedoMove:nil toPosition:redoMoveFloat withOpposingMoveValue:undoMoveFloat];
			}
		}
	}
	
@end
